/* 
   - The contents of this file are subject to the Mozilla Public
   - License Version 1.1 (the "License"); you may not use this file
   - except in compliance with the License. You may obtain a copy of
   - the License at http://www.mozilla.org/MPL/
   -
   - Software distributed under the License is distributed on an "AS
   - IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
   - implied. See the License for the specific language governing
   - rights and limitations under the License.
   -
   - The Original Code is this file as it was released on
   - August 21, 2003.
   -
   - The Initial Developer of the Original Code is Henk-Johan van Rantwijk.
   - Portions created by Henk-Johan van Rantwijk are Copyright (C) 2003-2004
   - Henk-Johan van Rantwijk.  All Rights Reserved.
   -
*/

const NO_ROWS_AVAILABLE      = -1;
const NO_ROWS_SELECTED       = 0;
const SINGLE_ROW_SELECTED    = 1;
const MULTIPLE_ROWS_SELECTED = 2;

var pref = null;
var prefWindow = null;
var prefPanelNames;
var managerTypes = [ "tabManager" ];
var dialogType = "tabManagerPrefWindow";
var panelPath = "chrome://multiviews/content/tab-manager/pref/";
var gUseStackedWindows = false;

/***
  * Toolbar button ID arrays that will be replaced with elements in updateToolbarButtons()
  * note: keep the order in-line with the corresponding XUL code!
  */
var toolbarButtons = [ "openButton", "importButton", "exportButton", "reloadButton", "propertiesButton", "closeButton", "prefsButton" ];
/***
  * Context Menu ID array that will be replaced with elements in initContextMenu(aEvent)
  * note: keep the order in-line with the corresponding XUL code!
  */
var contextMenuItems = [ "SelectAll", "UndoSelection", "SetInterval" ];
/***
  * Toolbar button arrays, to ease the pain of enabling/disabling the buttons
  */
var defaultEnabledButtons      = [  true, false, false, false,  true,  true, true ];
var selectionEnabledButtons    = [ false, false, false, false,  true,  true, true ];
var multipleRowsEnabledButtons = [ false, false, false, false, false,  true, true ];
var modifyEnabledButtons       = [ false, false,  true,  true, false, false, true ];

const DEFAULT_BUTTON = 0;
const CUSTOM_BUTTON  = 1;
const REPLACE_BUTTON = 3;

const CM_SELECT_ALL  = 0;
const CM_SELECT_NONE = 1;
const CM_DEFAULT     = 2;
const CM_CUSTOM      = 3;
const CM_ADD         = 4;
const CM_REPLACE     = 5;
const CM_EDIT        = 6;
const CM_REMOVE      = 7;

// Our global stuff goes here
var tabbrowser = null;
var stringBundle = null;
var busyTabs = -1;

var tree       = null;
var dataSource = null;

////////////////////////////////////////////////////////////////////////////////
function getNavigatorWindow(aOpenFlag)
{
  var windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService();
  var windowManagerInterface = windowManager.QueryInterface(Components.interfaces.nsIWindowMediator);
  var navigatorWindow = windowManagerInterface.getMostRecentWindow("navigator:browser");

  if (!navigatorWindow) {
    if (!aOpenFlag)
      return null;
    else {
      window.open();
      navigatorWindow = windowManagerInterface.getMostRecentWindow("navigator:browser");
    }
  }
  return navigatorWindow;
}
////////////////////////////////////////////////////////////////////////////////
function getTabbrowser(aOpenFlag)
{
  var navigatorWindow = getNavigatorWindow(aOpenFlag);

  if (navigatorWindow)
    return navigatorWindow.getBrowser();
  return null;
}
////////////////////////////////////////////////////////////////////////////////
function updateToolbarButtons()
{
  // Is the toolbar button array initialized?
  if (typeof toolbarButtons[0] == 'string') {
    // Nope, not yet, initialize array now
    for (i in toolbarButtons) {
      toolbarButtons[i] = document.getElementById(toolbarButtons[i]);
    }
  }
  var buttons = defaultEnabledButtons;
  var treeState = tabManager.getCurrentTreeState();

  if (treeState > NO_ROWS_SELECTED) {
    if (treeState == SINGLE_ROW_SELECTED)
      buttons = selectionEnabledButtons;
    else if (treeState == MULTIPLE_ROWS_SELECTED)
      buttons = multipleRowsEnabledButtons;
  }
  // Enable/disable the toolbar buttons based on the toolbar array
  for (i in toolbarButtons) {
    toolbarButtons[i].disabled = !buttons[i];
  }
}
////////////////////////////////////////////////////////////////////////////////
function initContextMenu(aEvent)
{
  // Is the context menu item array initialized?
  if (typeof contextMenuItems[0] == 'string') {
    // Nope, not yet, initialize array now
    for (i in contextMenuItems) {
      contextMenuItems[i] = document.getElementById(contextMenuItems[i]);
    }
  }
  for (i in contextMenuItems) {
    contextMenuItems[i].setAttribute('disabled', 'true');
  }
  var treeState = userAgentManager.getCurrentTreeState();

  if (treeState == NO_ROWS_AVAILABLE)
    return;

  var treeRows = tree.treeBoxObject.view.rowCount;

  if (treeState == MULTIPLE_ROWS_SELECTED) {
    contextMenuItems[CM_SELECT_ALL].setAttribute('disabled', 'true');
    contextMenuItems[CM_SELECT_NONE].removeAttribute('disabled');
  }
  else { // NO_ROWS_SELECTED || SINGLE_ROW_SELECTED
    if (treeRows > 1 && textField.value == "") 
      contextMenuItems[CM_SELECT_ALL].removeAttribute('disabled');
    contextMenuItems[CM_SELECT_NONE].setAttribute('disabled', 'true');
  }
}
//////////////////////////////////////////////////////////////////////////////
function initReloadPopup(aPopup, aEvent)
{
  /* if (toolbarButtons[3].disabled) {
    aEvent.preventDefault();
    return;
  } */
  tabbrowser.initReloadPopup(aPopup, null, tabManager.getReloadInterval());
}
////////////////////////////////////////////////////////////////////////////////
function openTMPropertyWindow()
{
  var treeState = tabManager.getCurrentTreeState();

  if (treeState == NO_ROWS_SELECTED || treeState == MULTIPLE_ROWS_SELECTED || currentRow == -1)
    return;

  var winFeatures = "chrome, dependent, modal, dialog" + (gUseStackedWindows ? "" : ", resizable=yes, centerscreen");
  window.openDialog("chrome://multiviews/content/tab-manager/tabProperties.xul", 
                    "tmproperties", winFeatures, window);
}
////////////////////////////////////////////////////////////////////////////////
function openPrefOverlay()
{
  var winFeatures = "chrome, dependent, modal, dialog" + (gUseStackedWindows ? "" : ", resizable=yes, centerscreen");
  window.openDialog("chrome://multiviews/content/tab-manager/tabManagerPrefWindow.xul", 
                    "prefWindow", winFeatures, window, "");
}
////////////////////////////////////////////////////////////////////////////////
// This is a dummy function, called from the pref manager (for compatibility only)
function switchTree(gPanel)
{
  return;
}
////////////////////////////////////////////////////////////////////////////////
function switchPanel(aPanel)
{
  var categoryButton = null;
  var panelFile = null;

  prefWindow.gSecondLevelPrefs = false;

  if (aPanel == "tabManagerPrefWindow")
    panelFile = "tabManagerMainPanel.xul";
  prefWindow.refreshPanel(panelFile, categoryButton);
}
////////////////////////////////////////////////////////////////////////////////
function initWindowOpenType()
{
  try {
    gUseStackedWindows = pref.getBoolPref("multizilla.managers.use-stacked-windows");
  } catch(ex) {
    gUseStackedWindows = /Win/.test(navigator.platform);
  }
}
////////////////////////////////////////////////////////////////////////////////
function buildTree()
{
  dataSource = Components.classes['@mozilla.org/rdf/datasource;1?name=in-memory-datasource'].createInstance(Components.interfaces.nsIRDFDataSource);
  tree = document.getElementById('tabbrowserTree');
  tree.database.AddDataSource(dataSource);
}
////////////////////////////////////////////////////////////////////////////////
function addTabbrowserTabs(aTabbrowser)
{
  addTabDataSeqToDS(dataSource, aResource, 0, aTabbrowser);
}
////////////////////////////////////////////////////////////////////////////////
function getTreerowByOrdinal(aOrdinal)
{
  var rows = tree.treeBoxObject.view.rowCount;
  var columnID = "tm-ordinal";
  columnID = ('columns' in tree) ? tree.columns[columnID] : columnID;

  for (var i = 0; i < rows; i++) {
    var treerow = document.getElementById("tm-tab-"+i+1);

    if (tree.view.getCellText(i, columnID) == aOrdinal) {
      return i;
    }
  }
  return -1;
}
////////////////////////////////////////////////////////////////////////////////
function tabListInit()
{
  var tabList = tabbrowser.mTabList;
  var tabs = tabList.length;
  var URL = null;
  // var windowtype = "window-0";

  for (var i = 0; i < tabs; i++) {
    var tab = tabList[i];
    var browser = tabbrowser.getBrowserForTab(tab);
    var favicon = tab.getAttribute("image");
    // Use user friendly formats for Reload Intervals like 3600/60000
    var reloadInterval = tab.mReloadInterval/1000 || 0;
    if (reloadInterval > 59)
      reloadInterval = reloadInterval/60 + " min";
    else reloadInterval = reloadInterval ? reloadInterval + " sec" : "";

    if (tab.hasAttribute("busy")) {
      busyTabs++;
      favicon = "chrome://multiviews/content/images/tabs/red.gif";
    }
    else if (!favicon)
      favicon = "chrome://multiviews/content/images/tabs/green.gif";
    var label = tab.getAttribute("label");

    try {
      URL = browser.webNavigation.currentURI.spec;
    } catch(ex) {
      URL = tab.getAttribute("firstpage");
    }
    createCell((i+1), favicon, label, URL, reloadInterval, tab.ordinal);
  }
  setTimeout(updateCurrentIndex, 0);
}
////////////////////////////////////////////////////////////////////////////////
function addContainer(aWindow)
{
  var treeKids = document.getElementById("tabList");
  var item     = document.createElement("treeitem");
  var children = document.createElement("treechildren");

  item.setAttribute("container", "true");
  item.setAttribute("open", "true");
  item.setAttribute("primary", "false");
  // item.setAttribute("label", aWindow);
  children.setAttribute("id", "window-0");
  item.appendChild(children);
  treeKids.appendChild(item);
}
////////////////////////////////////////////////////////////////////////////////
function createCell(aNumber, aImage, aLabel, aURL, aReloadInterval, aOrdinal)
{
  var treeKids = document.getElementById("tabList");
  var row      = document.createElement("treerow");
  var tab      = document.createElement("treecell");
  // var image    = document.createElement("treecell");
  var label    = document.createElement("treecell");
  var URL      = document.createElement("treecell");
  var interval = document.createElement("treecell");
  var ordinal  = document.createElement("treecell");
  var item     = document.createElement("treeitem");

  tab.setAttribute("label", aNumber);
  label.setAttribute("label", aLabel);
  label.setAttribute("src", aImage);
  // image.setAttribute("src", aImage);
  URL.setAttribute("label", aURL);
  interval.setAttribute("label", aReloadInterval);
  ordinal.setAttribute("label", aOrdinal);
  
  row.appendChild(tab);
  row.appendChild(label);
  // row.appendChild(image);
  row.appendChild(URL);
  row.appendChild(interval);
  row.appendChild(ordinal);

  item.setAttribute("id", "tm-tab-"+aNumber);
  item.appendChild(row);

  treeKids.appendChild(item);
}
////////////////////////////////////////////////////////////////////////////////
function updateCurrentIndex()
{
  tree.currentIndex = tabbrowser.selectedTab.ordinal;
  tree.view.selection.toggleSelect(tree.currentIndex);
}
////////////////////////////////////////////////////////////////////////////////
function activateSelectedTab()
{
  var tabs = getSelectedTreeItems();

  if (tabs && tabs.length == 1) {
    tabbrowser.selectedTab = tabs[0];
    tree.selectedItem = tabs[0].ordinal;
  }
  // setTimeout("tree.focus()", 0); // Hack to prevent main browser window to steal focus!
}
////////////////////////////////////////////////////////////////////////////////
function getSelectedTreeItems()
{
  var t = -1;
  var tabs = new Array();
  // Get and return all selection ranges
  var rangeCount = tree.view.selection.getRangeCount();
  var columnID = "tm-ordinal";
  columnID = ('columns' in tree) ? tree.columns[columnID] : columnID;
  // Iterate over all ranges to collect the selected items
  for (var i = 0; i < rangeCount; i++) {
    var startSelection = {};
    var endSelection = {};
    tree.view.selection.getRangeAt(i, startSelection, endSelection);
    // Collect all items and store them in our array
    for (var l = startSelection.value; l <= endSelection.value; l++) {
      var index = tree.view.getCellText(l, columnID);
      tabs[++t] = tabbrowser.mTabList[index];
      // dump("range: "+i+"loop: "+l+"tab: "+t"\n");
    }
  }
  if (tabs)
    return tabs;
  return null;
}
////////////////////////////////////////////////////////////////////////////////
function doRemove(aSelection)
{
  var confirmationType = (aSelection == "all") ? "multiple" : "single";
  // Do we have to do select items first (this is done from the context menu)?
  if (aSelection)
    doSelect(aSelection);
  // Collect the selected items
  var tabs = getSelectedTreeItems();
  // Do we have a selected tab?
  if (tabs && tabs.length > 0) {
    // Ask for confirmation before performing the destructive task
    if (!confirmClose(confirmationType)) {
      tree.focus();
      return;
    }
    // Confirmed by user, close the tab(s) one after one
    for (var i = (tabs.length-1); i > -1; i--) {
      tabbrowser.removeTab(tabs[i]);
    }
    tabs = null;
    // window.close(); // must be replace with some sort of update window tree thing!
  }
}
////////////////////////////////////////////////////////////////////////////////
function closeTab()
{
  if (confirmClose("single")) {
    tabbrowser.removeTab(tabbrowser.selectedTab);
    window.close(); // must be replace with some sort of update window tree thing!
  }
}
////////////////////////////////////////////////////////////////////////////////
function confirmClose(allFlag)
{
  var confirmationState = false;
  var confirmDialogTitle = (allFlag == "single") ? "tab.confirmDialogTitle" : "tabs.confirmDialogTitle";
  var confirmDialogText = (allFlag == "single") ? "tab.confirmDialogText" : "tabs.confirmDialogText";

  if (tabbrowser.confirm(confirmDialogTitle, confirmDialogText))
    confirmationState =  true;
  tree.focus();
  return confirmationState;
}
/* function initContextMenu()
{
  var rows = tree.treeBoxObject.view.rowCount;
  var selectedItems = getSelectedTreeItems();

  // Disable "Select All" when there's a selection - note: "Close All Tabs use <observes/> to synchronize
  // Disable "Deselect All" when there's no selection
  var selectAll = document.getElementById("tmc-select-tabs");
  var deselectAll = document.getElementById("tmc-undoSelection");
  if (selectedItems.length > 1) {
    selectAll.setAttribute("disabled", "true");
    deselectAll.removeAttribute("disabled");
  }
  else { // No selection, enable "Select All" and disable "Deselect All"
    if (rows > 1)
      selectAll.removeAttribute("disabled");
    deselectAll.setAttribute("disabled", "true");
  }
  // Disable "Reload Tab Every" for blank tabs
  var browser = tabbrowser.getBrowserForTab(tabbrowser.selectedTab);
  var autoReloadMenu = document.getElementById("tmc-set-interval");
  if (browser.currentURI.spec == "about:blank")
    autoReloadMenu.setAttribute("disabled", "true");
  else autoReloadMenu.removeAttribute("disabled");

  // Display/hide reload indicator
  if (tabbrowser.selectedTab.mReloadInterval)
    autoReloadMenu.setAttribute("tab-will-reload", "true");
  else
    autoReloadMenu.removeAttribute("tab-will-reload");

  // Disable "Close All Tabs" when there's only one tab
  var closeAllTabs = document.getElementById("tmc-close-all-tabs");
  if (rows > 1)
    closeAllTabs.removeAttribute("disabled");
  else closeAllTabs.setAttribute("disabled", "true");
} */

////////////////////////////////////////////////////////////////////////////////
var tabManager = 
{
  tree : null,
  currentRow : null,
  stringBundle : null,
  tabbrowser : null,

  initTabManager: function()
  {
    tree = document.getElementById('tabManagerTree');
    mzRDFStore = getNewOrCurrentDSByFilename("useragents.rdf", true, null, false);
    tree.database.AddDataSource(mzRDFStore);
    tree.builder.rebuild();
    stringBundle = document.getElementById('properties');
    tabbrowser = getTabbrowser(true);
    currentRow = -1;

    var prefService = Components.classes["@mozilla.org/preferences-service;1"]
                                .getService(Components.interfaces.nsIPrefService);
    pref = prefService.getBranch(null);
    prefPanelNames = stringBundle.getString("prefPanelNames").split(',');

    sizeToContent();
    initWindowOpenType();
    updateToolbarButtons();
  },

  getRowNumber: function(aEvent)
  {
    var X = aEvent.pageX;
    var Y = aEvent.pageY;
    return tree.treeBoxObject.getRowAt(X, Y);
  },

  getSelectedRows: function()
  {
    var rows = -1;
    var selectedRows = new Array();
    // Get and return all selection ranges
    var rangeCount = tree.builderView.selection.getRangeCount();
    // Iterate over all ranges to collect the selected rows
    for (var range = 0; range < rangeCount; range++) {
      var startSelection = {};
      var endSelection = {};
      tree.builderView.selection.getRangeAt(range, startSelection, endSelection);
      // Collect all rows and store them in our array
      for (var row = startSelection.value; row <= endSelection.value; row++) {
        // Check for invalid row numbers
        if (startSelection.value != -1 || endSelection.value != -1)
          selectedRows[++rows] = row;
      }
    }
    if (selectedRows)
      return selectedRows;
    return null;
  },

  getCurrentTreeState: function()
  {
    if (tree.treeBoxObject.view.rowCount < 1)
      return NO_ROWS_AVAILABLE;

    var selectedRows = this.getSelectedRows().length;

    if (selectedRows > 1)
      return MULTIPLE_ROWS_SELECTED;
    else if (selectedRows == 1)
      return SINGLE_ROW_SELECTED;
    return NO_ROWS_SELECTED;
  },

  getReloadInterval: function(aEvent)
  {
    var interval = 0;
    var columnID = "tm-interval";
    columnID = ('columns' in tree) ? tree.columns[columnID] : columnID;

    if (aEvent) {
      var row = this.getRowNumber(aEvent);
      interval = (row > -1) ? tree.view.getCellText(row, columnID) : 0;
    }
    else if (!aEvent && tree.currentIndex != -1)
      interval = tree.view.getCellText(tree.currentIndex, columnID);

    if (interval)
      return interval;
    return 1;
  },

  setReloadInterval: function(aEvent, aValue)
  {
    var resource = this.getResourceForRow(currentRow);
    var intervalProp = RDF.GetResource("http://multizilla.mozdev.org/rdf#interval");
    var currentValue = mzRDFStore.GetTarget(RDF.GetResource(resource), intervalProp, true);

    if (aValue == 0) // Remove interval property from tree
      mzRDFStore.Unassert(RDF.GetResource(resource), intervalProp, currentValue);
    else
      addOrChangeValue(mzRDFStore, RDF.GetResource(resource), intervalProp, currentValue, RDF.GetLiteral(aValue));
  },

  selectRows: function(aCommand)
  {
    switch (aCommand) {
      case "all": // Called from the context menu, or Ctrl+A ?
           tree.view.selection.selectAll();
           break;
      case "none": // Called from the context menu, or Ctrl+Z ?
           tree.view.selection.clearSelection();
           tree.view.selection.select(tree.currentIndex);
           break;
    }
    tree.focus();
  },

  onSelect: function()
  {
    currentRow = tree.currentIndex;
    document.documentElement.getButton("accept").removeAttribute("disabled");
    updateToolbarButtons();
    activateSelectedTab();
  },

  onHelp: function()
  {
    var helpPages = stringBundle.getString("helpPages-TabManager");
    // We could eventually check the on-line state!
    tabbrowser.loadURI(helpPages);
    window.close();
  },

  onCancel: function()
  {
    window.close();
  },

  onAccept: function()
  {
    var tabs = this.getSelectedTreeItems();

    if (tabs && tabs.length > 1)
      doRemove();
    else
      activateSelectedTab();
    // window.close();
  }
};

addEventListener("load", tabManager.initTabManager, false);

/* var tabManagerDNDObserver = 
{
  tree : null,
  
  initTabManagerDNDObserver: function()
  {
    tree = document.getElementById("tabManagerTree");
    initializeRDFStorageFile();
    tree.database.AddDataSource(mzRDFStore);
    tree.builder.rebuild();
    updateCurrentIndex();
    tabbrowser = getLastActiveBrowserWindow().getBrowser();
    stringBundle = document.getElementById("tm-properties");
    tabListInit();
  },

  onDragStart: function(aEvent, aXferData, aDragSession)
  {
    // start the row drag operation only when there are at least 2 visible rows
    if (tree.treeBoxObject.view.rowCount > 1) {
      aEvent.target.setAttribute("dragover", "true");
      var columnID = "sm-interval";
      columnID = ('columns' in tree) ? tree.columns[columnID] : columnID;
      var ordinal = tree.view.getCellText(tree.currentIndex, columnID);
      // dump("onDragStart ordinal: "+ordinal+"\n");
      aXferData.data = new TransferData();
      aXferData.data.addDataForFlavour("text/unicode", ordinal);
    }
  },

  onDragOver: function(aEvent, aXferData, aDragSession)
  {
    var X = aEvent.pageX;
    var Y = aEvent.pageY;
    var row = tree.treeBoxObject.getRowAt(X, Y);

    aDragSession.canDrop = true;
    if (row > -1) {
      dump("onDragOver - row: "+row+"\n");
      var columnID = "sm-interval";
      columnID = ('columns' in tree) ? tree.columns[columnID] : columnID;
      var ordinal = tree.view.getCellText(row, columnID);
      dump("ordinal: "+ordinal+"\n");
      ordinal++;
      var element = document.getElementById("tm-tab-"+ordinal);
      dump(element.nodeName+"\n");
      // nsDragAndDrop.mDragSession.sourceNode = element;
    }
  },

  onDrop: function(aEvent, aXferData, aDragSession)
  {
    dump("onDrop\n"+aEvent.originalTarget.nodeName);
  },

  onDragExit: function(aEvent, aXferData, aDragSession)
  {
    tree.childNodes[1].removeAttribute("dragover");
  },

  getSupportedFlavours: function ()
  {
    // we might want to expand this
    var flavourSet = new FlavourSet();
    flavourSet.appendFlavour("text/unicode");
    return flavourSet;
  },

  canDrop: function (aEvent, aDragSession)
  {
    aDragSession.canDrop = true;
    // allow drops from the Tab Manager only
    var doc = aDragSession.sourceNode.ownerDocument;
    var elem = doc.getElementById("tabManagerWindow");
    return (elem && (elem.getAttribute("windowtype") == "multizilla:tabmanager"));
  } 
};
addEventListener("load", tabManagerDNDObserver.initTabManagerDNDObserver, false); */